@@ -1,12 +1,12 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
-__version__ = '1.0.3'
+__version__ = '1.0.4-1'
 
 import sys
 
-if sys.version_info < (3, 8, 1):
-    raise RuntimeError('Requires Python version 3.8.1 or higher. This version: ' + str(sys.version_info))
+if sys.version_info < (3, 6, 1):
+    raise RuntimeError('Requires Python version 3.6.1 or higher. This version: ' + str(sys.version_info))
 
 usage = "Usage: " + sys.argv[0] + " option" + """
 
@@ -18,14 +18,26 @@ usage = "Usage: " + sys.argv[0] + " option" + """
         label|set  <s3bucket>/<s3object> '{"label":"value"}'
         delete|del <s3bucket>/<s3object> label
 
+        browser    <s3bucket>/<s3object>
+        view       <s3bucket>/<s3object>
+
         get    <s3bucket>/<s3object>
         save   <s3bucket>/<s3object> destination
         upload source <s3bucket>/<s3object>
 
         rekognition <s3bucket>/<s3object>
         rekognition <s3bucket>/<s3object> detect-labels
+
         rekognition <s3bucket>/<s3object> words
-        rekognition <s3bucket>/<s3object> s3tag
+        rekognition <s3bucket>/<s3object> s3tag words
+
+        rekognition <s3bucket>/<s3object> confidence
+        rekognition <s3bucket>/<s3object> confidence top 3
+        rekognition <s3bucket>/<s3object> confidence top 90 percent
+
+        rekognition <s3bucket>/<s3object> s3tag confidence
+        rekognition <s3bucket>/<s3object> s3tag confidence top 3
+        rekognition <s3bucket>/<s3object> s3tag confidence top 90 percent
 
         object      <s3bucket>/<s3object>
         b2sum       <s3bucket>/<s3object>
@@ -36,6 +48,7 @@ usage = "Usage: " + sys.argv[0] + " option" + """
         --help
         --version
 """
+        # browser    <s3bucket>/<s3object>
 
 import boto3
 import botocore
@@ -44,6 +57,8 @@ import json
 
 from hashlib import blake2b, blake2s
 
+#import webbrowser
+
 from flask import Flask
 from flask import request
 from flask import jsonify
@@ -102,6 +117,8 @@ def get_s3bucketobject(s3bucket=None,s3object=None):
     delete      = request.args.get("delete", None)
     label       = request.args.get("label", None)
     value       = request.args.get("value", None)
+    top         = request.args.get("top", None)
+    percent     = request.args.get("percent", None)
 
     s3_client = boto3.client('s3')
 
@@ -194,6 +211,8 @@ def get_s3bucketobject(s3bucket=None,s3object=None):
 
     if rekognition:
 
+        ######################################################################################################################################
+
         if rekognition == 'detect-labels':
 
             #either specify region or auto get region from boto call
@@ -227,6 +246,7 @@ def get_s3bucketobject(s3bucket=None,s3object=None):
             else:
                 return jsonify(response), 200
 
+        ######################################################################################################################################
 
         if rekognition == 'json':
             rekognition_json_file = 'rekognition/' + s3object + '.json'
@@ -236,6 +256,8 @@ def get_s3bucketobject(s3bucket=None,s3object=None):
             else:
                 return jsonify(status=404, message="Not Found", s3object=False, rekognition_json_location=rekognition_json_file), 404
 
+        ######################################################################################################################################
+
         if rekognition == 'words':
 
             rekognition_json_file = 'rekognition/' + s3object + '.json'
@@ -282,6 +304,84 @@ def get_s3bucketobject(s3bucket=None,s3object=None):
             else:
                 return jsonify(status=404, message="Not Found", s3object=False, rekognition_json_location=rekognition_json_file), 404
 
+        ######################################################################################################################################
+
+        if rekognition == 'confidence':
+
+            rekognition_json_file = 'rekognition/' + s3object + '.json'
+            rekognition_json_content = get_s3object_body(s3bucket, rekognition_json_file)
+
+            if rekognition_json_content:
+
+                #wordList = extract_rekognition_words(rekognition_json_content)
+
+                s3 = boto3.resource('s3')
+                obj = s3.Object(s3bucket, rekognition_json_file)
+
+                try:
+                    body = obj.get()['Body'].read()
+                except botocore.exceptions.ClientError as e:
+                    if e.response['Error']['Code'] == 'NoSuchKey':
+                        return jsonify(status=404, message="Not Found", s3object=False, key=rekognition_json_file), 404
+                    else:
+                        return jsonify(status=599, message="ClientError", s3object=False, key=rekognition_json_file, error=str(e)), 599
+
+                content = body.decode("utf-8", "strict").rstrip()
+                data = json.loads(content)
+
+
+                Dict={}
+
+                ############################################################################################################################
+                if top:
+
+                    if percent:
+
+                        for key in data['Labels']:
+                            _Name = key['Name']
+                            _Confidence = key['Confidence']
+                            if int(top) <= int(_Confidence):
+                                Dict[_Name]= str(_Confidence)
+                    else:
+
+                        count=0
+                        for key in data['Labels']:
+                            count += 1
+                            if count <= int(top):
+                                _Name = key['Name']
+                                _Confidence = key['Confidence']
+                                Dict[_Name]= str(_Confidence)
+
+                else:
+                    for key in data['Labels']:
+                        _Name = key['Name']
+                        _Confidence = key['Confidence']
+                        Dict[_Name]= str(_Confidence)
+
+
+                ############################################################################################################################
+                if save == 's3tag':
+
+                    updated=0
+                    for k,v in Dict.items():
+                        try:
+                            update = update_s3object_tag(s3bucket, s3object, k, v)
+                            updated += 1
+                        except botocore.exceptions.ClientError as e:
+                            return jsonify(status=465, message="Failed S3Tag", label=False, error=str(e)), 465
+
+                    if updated > 0:
+                        return jsonify(status=201, message="Created S3Tag", label=True), 201
+                    else:
+                        return jsonify(status=465, message="Failed S3Tag", label=False), 465
+
+                return jsonify(Dict), 200
+                ############################################################################################################################
+            else:
+                return jsonify(status=404, message="Not Found", s3object=False, rekognition_json_location=rekognition_json_file), 404
+
+        ######################################################################################################################################
+
     if delete:
 
         delete_tag = delete_s3object_tag(s3bucket, s3object, delete)
@@ -718,9 +818,15 @@ def main():
         if sys.argv[1] == "--help":
             sys.exit(print(usage))
 
+        ##############################################################################################
+        ##############################################################################################
+
         if sys.argv[1] == "--version":
             sys.exit(print(__version__))
 
+        ##############################################################################################
+        ##############################################################################################
+
         if sys.argv[1] == "buckets" or sys.argv[1] == "list-buckets":
 
             try:
@@ -731,6 +837,9 @@ def main():
             
             sys.exit(print(json.dumps(buckets, indent=2)))
 
+        ##############################################################################################
+        ##############################################################################################
+
         if sys.argv[1] == "del" or sys.argv[1] == "delete":
             s3path = sys.argv[2]
             tag    = sys.argv[3]
@@ -750,6 +859,9 @@ def main():
                 print(json.dumps({'delete':False}))
                 sys.exit(1)
 
+        ##############################################################################################
+        ##############################################################################################
+
 
         if sys.argv[1] == "set" or sys.argv[1] == "label":
 
@@ -783,6 +895,8 @@ def main():
                 print(json.dumps({'label':False}))
                 sys.exit(1)
             
+        ##############################################################################################
+        ##############################################################################################
 
         if sys.argv[1] == "ls" or sys.argv[1] == "list":
             s3path = sys.argv[2]
@@ -839,6 +953,8 @@ def main():
             print(json.dumps(s3Tags, indent=2))
             sys.exit(0)
 
+        ##############################################################################################
+        ##############################################################################################
 
 
         if sys.argv[1] == "get":
@@ -856,6 +972,9 @@ def main():
             print(content.rstrip())
             sys.exit(0) 
 
+        ##############################################################################################
+        ##############################################################################################
+
         if sys.argv[1] == "save":
 
             s3path      = sys.argv[2]
@@ -886,6 +1005,8 @@ def main():
             print(json.dumps({'saved':destination}, indent=2))
             sys.exit(0)
 
+        ##############################################################################################
+        ##############################################################################################
 
         if sys.argv[1] == "rekognition":
 
@@ -898,6 +1019,8 @@ def main():
             try: s3object = s3path.split("/", 1)[1]
             except IndexError: s3object = ''
 
+            ##############################################################################################
+
             if option == 's3tag':
 
                 rekognition_json_file = 'rekognition/' + s3object + '.json'
@@ -917,24 +1040,105 @@ def main():
 
                 data = json.loads(content)
 
-                List=[]
-                for key in data['Labels']:
-                    List.append(key['Name'])
+                try: option4 = sys.argv[4] 
+                except IndexError: option4 = None
 
-                listToStr = ' '.join([str(elem) for elem in List])
+                if option4:
+                    # words||confidence
 
-                tag = 'rekognition-words'
+                    if option4 == 'words':
+                        #print('words')
+                        List=[]
+                        for key in data['Labels']:
+                            List.append(key['Name'])
+                        listToStr = ' '.join([str(elem) for elem in List])
+                        tag = 'rekognition-words'
+                        update = update_s3object_tag(s3bucket, s3object, tag, listToStr)
+                        if update == True:
+                            print(json.dumps({'label':True}))
+                            sys.exit(0)
+                        else:
+                            print(json.dumps({'label':False}))
+                            sys.exit(1)
 
-                update = update_s3object_tag(s3bucket, s3object, tag, listToStr)
 
-                if update == True:
-                    print(json.dumps({'label':True}))
-                    sys.exit(0)
+                    if option4 == 'confidence':
+
+                        try: option5 = sys.argv[5] 
+                        except IndexError: option5 = None
+
+                        try: option6 = sys.argv[6] 
+                        except IndexError: option6 = None
+
+                        try: option7 = sys.argv[7] 
+                        except IndexError: option7 = None
+
+                        #print(str(option5)) #top
+                        #print(str(option6)) #3 number
+                        #print(str(option7)) #percent
+
+                        Dict={}
+
+                        if option5 == 'top' and option7 == None:
+
+                            count=0
+
+                            for key in data['Labels']:
+                                count += 1
+                                if count <= int(option6):
+                                    _Name = key['Name']
+                                    _Confidence = key['Confidence']
+                                    Dict[_Name]= str(_Confidence)
+
+
+                        elif option5 == 'top' and option7 == 'percent':
+
+                            for key in data['Labels']:
+                                _Name = key['Name']
+                                _Confidence = key['Confidence']
+
+                                if int(option6) <= int(_Confidence):
+                                    Dict[_Name]= str(_Confidence)
+
+                        else:
+
+                            for key in data['Labels']:
+                                _Name = key['Name']
+                                _Confidence = key['Confidence']
+                                Dict[_Name]= str(_Confidence)
+
+                        ###########################################################
+
+                        updated=0
+                        for k,v in Dict.items():
+                            try:
+                                update = update_s3object_tag(s3bucket, s3object, k, v)
+                                updated += 1
+                            except botocore.exceptions.ClientError as e:
+                                print(json.dumps({'label':False, 'error': str(e), 's3tag': str(k)}))
+                                sys.exit(1)
+
+                        if updated > 0:
+                            print(json.dumps({'label':True}))
+                            sys.exit(0)
+                        else:
+                            print(json.dumps({'label':False}))
+                            sys.exit(1)
+
+                        sys.exit(1)
+
+                    else:
+                        print('Unknown option ' + option4)
+                        sys.exit(1)
+
                 else:
-                    print(json.dumps({'label':False}))
+                    print('options are words or confidence')
                     sys.exit(1)
 
 
+                sys.exit(0)
+
+            ##############################################################################################
 
             if option == 'words':
 
@@ -962,7 +1166,71 @@ def main():
                 print(json.dumps(List, indent=2))
                 sys.exit(0)
 
+            ##############################################################################################
 
+            if option == 'confidence':
+
+                try: option4 = sys.argv[4] 
+                except IndexError: option4 = None
+
+                try: option5 = sys.argv[5] 
+                except IndexError: option5 = None
+
+                try: option6 = sys.argv[6] 
+                except IndexError: option6 = None
+
+                #print(str(option4)) #top
+                #print(str(option5)) #3 number
+                #print(str(option6)) #percent
+
+                rekognition_json_file = 'rekognition/' + s3object + '.json'
+                s3 = boto3.resource('s3')
+                obj = s3.Object(s3bucket, rekognition_json_file)
+                try:
+                    body = obj.get()['Body'].read()
+
+                except botocore.exceptions.ClientError as e:
+                    if e.response['Error']['Code'] == 'NoSuchKey':
+                        print(json.dumps({'NoSuchKey':rekognition_json_file}, indent=2))
+                    else:
+                        print(json.dumps({'ClientError':str(e)}))
+                    sys.exit(1)
+
+                content = body.decode("utf-8", "strict").rstrip()
+
+                data = json.loads(content)
+
+                Dict={}
+
+                if option4 == 'top' and option6 == None:
+                    count=0
+                    for key in data['Labels']:
+                        count += 1
+                        if count <= int(option5):
+                            _Name = key['Name']
+                            _Confidence = key['Confidence']
+                            Dict[_Name]= str(_Confidence)
+
+
+                elif option4 == 'top' and option6 == 'percent':
+                    for key in data['Labels']:
+                        _Name = key['Name']
+                        _Confidence = key['Confidence']
+
+                        if int(option5) <= int(_Confidence):
+                            Dict[_Name]= str(_Confidence)
+
+                else:
+
+                    for key in data['Labels']:
+                        _Name = key['Name']
+                        _Confidence = key['Confidence']
+                        Dict[_Name]=_Confidence
+
+                print(json.dumps(Dict, indent=2))
+                sys.exit(0)
+
+            ##############################################################################################
 
             if option == 'detect-labels':
 
@@ -1016,6 +1284,8 @@ def main():
             print(content)
             sys.exit(0)
 
+        ##############################################################################################
+        ##############################################################################################
 
         if sys.argv[1] == "object":
             s3path = sys.argv[2]
@@ -1055,6 +1325,9 @@ def main():
             print(json.dumps(Objects, indent=2, sort_keys=True, default=str))
             sys.exit(0)
 
+        ##############################################################################################
+        ##############################################################################################
+
         if sys.argv[1] == "identify" or sys.argv[1] == "id":
 
             s3path = sys.argv[2]
@@ -1120,6 +1393,9 @@ def main():
             print(json.dumps(Objects, indent=2, sort_keys=True, default=str))
             sys.exit(0)
 
+        ##############################################################################################
+        ##############################################################################################
+
         if sys.argv[1] == "b2sum":
 
             s3path = sys.argv[2]
@@ -1175,6 +1451,9 @@ def main():
             print(json.dumps(Objects, indent=2, sort_keys=True, default=str))
             sys.exit(0)
 
+        ##############################################################################################
+        ##############################################################################################
+
 
         if sys.argv[1] == "upload":
 
@@ -1195,6 +1474,107 @@ def main():
             print(json.dumps({'upload':True}, indent=2))
             sys.exit(0)
 
+        ##############################################################################################
+        ##############################################################################################
+
+        if sys.argv[1] == "browser":
+            
+            s3path = sys.argv[2]
+            s3bucket = s3path.split("/", 1)[0]
+            s3object = s3path.split("/", 1)[1]
+
+            s3 = boto3.client('s3')
+
+            from tempfile import mkstemp
+
+            suffix = str(s3object).replace('/', '_')
+
+            fd, path = mkstemp(suffix=suffix)
+
+            try:
+                s3.download_file(s3bucket, s3object, path)
+
+            except botocore.exceptions.ClientError as e:
+                if e.response['Error']['Code'] == '404':
+                    print(json.dumps({'Not Found':s3object}, indent=2))
+                else:
+                    print(json.dumps({'ClientError':str(e)}, indent=2))
+                sys.exit(1)
+
+            import webbrowser
+            webbrowser.open('file://' + str(path))
+
+            print(json.dumps({'tempfile': path}))
+            sys.exit(0)
+
+        ##############################################################################################
+        ##############################################################################################
+
+        if sys.argv[1] == "view":
+
+            s3path = sys.argv[2]
+            s3bucket = s3path.split("/", 1)[0]
+            s3object = s3path.split("/", 1)[1]
+
+            s3 = boto3.client('s3')
+
+            suffix = str(s3object).replace('/', '_')
+
+            from tempfile import mkstemp
+            fd, path = mkstemp(suffix=suffix)
+
+            try:
+                s3.download_file(s3bucket, s3object, path)
+
+            except botocore.exceptions.ClientError as e:
+                if e.response['Error']['Code'] == '404':
+                    print(json.dumps({'Not Found':s3object}, indent=2))
+                else:
+                    print(json.dumps({'ClientError':str(e)}, indent=2))
+                sys.exit(1)
+
+            import os
+
+            if sys.platform == 'darwin':
+                os.system("open " + str(path))
+                print(json.dumps({'tempfile': path}))
+                sys.exit(0)
+
+            elif sys.platform == 'linux' or sys.platform == 'linux2':
+                os.system("xdg-open tmp.png")
+                print(json.dumps({'tempfile': path}))
+                sys.exit(0)
+
+            elif sys.platform == 'win32' or sys.platform == 'win64':
+                os.system("powershell -c tmp.png")
+                print(json.dumps({'tempfile': path}))
+                sys.exit(0)
+
+            elif sys.platform == 'cygwin':
+                os.system("")
+                print(json.dumps({'tempfile': path, 'cygwin':'unknown command line'}))
+                sys.exit(1)
+
+            else:
+                print(json.dumps({'tempfile': path, 'unknown':str(sys.platform)}))
+                sys.exit(1)
+
+
+            #mac
+            #os.system("open tmp.png")
+
+            #linux
+            #os.system("xdg-open tmp.png")
+
+            #win
+            #os.system("powershell -c tmp.png")
+
+            print(json.dumps({'tempfile': path}))
+            sys.exit(0)
+
+        ##############################################################################################
+        ##############################################################################################
+
 
         if sys.argv[1] == "server":
             try:
